require 'rabbit/element'
require 'rabbit/parser/ext/video'

module Rabbit
  module Parser
    class RD
      module Ext
        module Video
          include Parser::Ext::Video
        end
      end
    end
  end
end

