@category = N_("Toolkit")
@title = N_("HeadlineLogo")
@abstract = N_("Toolkit to display an image as a logo at the headlines")
@description = N_("Displays an image as a logo at the headlines of slides.")
@parameters = {
  "@headline_logo_image" => {
    :default => N_("(Must be specified.)"),
    :description => N_("Image file name."),
  },
}
